/*
 * Copyright (c) 2024, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _HCS_CONTROL_DRIVER__H
#define _HCS_CONTROL_DRIVER__H

#include <stdint.h>
#include <math.h>
#include <stdbool.h>

/* This header will automatically pick out the correct device header */
#include <tps_hcsdev.h>

/**
 * @file hcs_control_driver.h
 * @brief Contains top level documentation for smart fuse drivers for the
 *             HCS family of high-side switches
 */

/* Response Status Header */
typedef union
{
    uint8_t byte;
    struct
    {
        unsigned I2T_FLT : 1;
        unsigned LPM_FLT : 1;
        unsigned CHAN_TSD : 1;
        unsigned ILIMIT_FLT : 1;
        unsigned SHRT_VBB_FLT : 1;
        unsigned OL_FLT : 1;
        unsigned SUPPLY_FLT : 1;
        unsigned GLOBAL_ERR_WRN : 1;
    } bits;
} tHCSResponseCode;

/* These are device specific definitions. Likely they are the same across
    all devices, but they are defined here for good measure. */
#define HCS_ADCREF 2.8f

/* --------------------------- Porting Functions ---------------------------- */
/*
 * These functions should be implemented by each individual device port. They
 * handle the low-level hardware specific implementation with the respective
 * architecture's specific hardware peripherals (SPI and GPIO)
 */
bool HCS_port_spiSendData(uint8_t *data, uint8_t len, uint8_t* respData);
void HCS_port_assertSPI(void);
void HCS_port_deassertSPI(void);

/* -------------------------- Daisy Chain Function -------------------------- */
// #define HCS_ENABLE_DAISY_CHAIN
#define HCS_startDaisyChain() HCS_port_assertSPI()
#define HCS_port_deassertSPI() HCS_port_assertSPI()

/* -------------------------- Internal Definitions -------------------------- */
#define HCS_PACKET_SIZE 3

/* Header used for write commands */
typedef union sWriteHeader
{
    uint8_t byte;
    struct
    {
        unsigned addr : 7;
        unsigned rw : 1;
    } bits;
} tWriteHeader;

typedef struct sHCSResponseStatus
{
    unsigned addr : 7;
    unsigned rw : 1;
} tHCSResponseStatus;

typedef struct sReadResponse
{
    uint8_t fltStatus;
    uint16_t payload;
} tReadResponse;

/* ---------------------------- Public Functions ---------------------------- */

/**
 *  @brief      Wakes up the device from sleep mode
 *
 *  This API simply issues a dummy write to register address 0xFF (which does
 *  not exist) so that the device wakes from sleep mode. Since the device wakes
 *  from sleep mode via a CS pin transition, the dummy write causes the device
 *  to wake up.
 *
 *  @retval returnCode An instance of tHCSResponseCode
 *
 */
tHCSResponseCode HCS_wakeupDevice(void);

/**
 *  @brief      Initializes the device with device configuration structure
 *
 *  This function will take a configuration structure that is generated from
 *  the Smart Fuse Configurator GUI software and send all of the values to
 *  the attached high-side switch. The common practice here would be to
 *  call this function on initial boot of the MCU or when the device wakes
 *  up from sleep mode with all of the register configurations lost.
 *
 *  @param[in]  config     Pointer to configuration structure
 *
 *  @retval returnCode An instance of tHCSResponseCode
 *
 */
tHCSResponseCode HCS_initializeDevice(TPS2HCS10Q1_CONFIG* config);

/**
 *  @brief      Updates the provided configuration with smart fuse settings
 *
 *  This API will populate the provided configuration structure with the values
 *  from the high-side switch. The could be used when the host software changes
 *  settings within the register and wants to update the initial configuration
 *  structure.
 *
 *  @param[out]  config     Pointer to configuration structure
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_updateConfig(TPS2HCS10Q1_CONFIG* config);

/**
 *  @brief      Sets the on/off switch state status per channel
 *
 *  This will turn on or off a channel of the high-side switch. The parameter
 *  is a bitwise representation of the channels to be enabled.
 *
 *  @param[in]  swState     Pointer to configuration structure
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_setSwitchState(uint8_t swState);

/**
 *  @brief      Performs a raw register write for specified register address
 *
 *  This function performs a simple register write to the given address with
 *  the provided payload
 *
 *  @param[in]  addr     Register address to write
 *  @param[in]  payload  Value to write to addr
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_writeRegister(uint8_t addr, uint16_t payload);

/**
 *  @brief      Performs a raw register read for specified register address
 *
 *  This function performs a simple register read from the given address and
 *  populates the provided pointer with the register contents.
 *
 *  @param[in]   addr     Register address to read
 *  @param[out]  payload  Value of register populated to this parameter
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_readRegister(uint8_t addr, uint16_t* readValue);

/**
 *  @brief      Puts the device into SLEEP mode
 *
 *  This function will send the command to go into SLEEP mode. Note that all
 *  values in the device's registers will be reset after exiting sleep mode. The
 *  device should be woken up by the HCS_wakeupDevice function.
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_gotoSleep(void);

/**
 *  @brief      Puts the device into LPM mode
 *
 *  This function will send the command to go into LPM mode. It is the MCU's
 *  responsibility to monitor the FLT pin for a falling edge if the exit current
 *  if the device exceeds the exit current level. Alternatively, the
 *  HCS_wakeupDevice function can be used to wakeup the device.
 *
 *  @param[in]   ch1ExitCurrent     Exit current for channel 1
 *  @param[in]   ch2ExitCurrent     Exit current for channel 2
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_gotoLPM(lpm_exit_curr_ch1_t ch1ExitCurrent,
                              lpm_exit_curr_ch2_t ch2ExitCurrent);

/**
 *  @brief      Converts a raw ADC voltage value to a read float value
 *
 *  This is a convenience function that will take a raw value read from
 *  one of the ADC result registers and convert it into a human readable
 *  float value.
 *
 *  @param[in]   rawValue    Raw value to convert
 *
 *  @retval returnCode Floating point representation of voltage
 */
float_t HCS_convertVoltage(uint16_t rawValue);

/**
 *  @brief      Converts a raw ADC current value to a readable float value
 *
 *  This is a convenience function that will take a raw value read from
 *  one of the ADC result registers and convert it into a human readable
 *  float value.
 *
 *  @param[in]   rawValue    Raw value to convert
 *  @param[in]   ksnsVal     KSNS constant from the datasheet
 *  @param[in]   snsRes      Value of resistor on the SNS pin
 *
 *  @retval returnCode Floating point representation of current
 */
float_t HCS_convertCurrent(uint16_t rawValue, uint16_t ksnsVal,
                            uint16_t snsRes);

/**
 *  @brief      Converts a raw ADC temperature value to a readable float value
 *
 *  This is a convenience function that will take a raw value read from
 *  one of the ADC result registers and convert it into a human readable
 *  float value.
 *
 *  @param[in]   rawValue    Raw value to convert
 *
 *  @retval returnCode Floating point representation of temperature
 */
float_t HCS_convertTemperature(uint16_t rawValue);

/**
 *  @brief      Reads an individual channel's fault status
 *
 *  Reads the individual channel's fault status and stores it into fltStatus
 *
 *  @param[in]   chanNum        Channel number to read
 *  @param[out]  fltStatus     Fault status will be stored here
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_getChannelFaultStatus(uint8_t chanNum,
                                           uint16_t *fltStatus);

/**
 *  @brief      Reads the device's fault status
 *
 *  Reads the device's fault status and stores it into fltStatus
 *
 *  @param[out]  fltStatus     Fault status will be stored here
 *
 *  @retval returnCode An instance of tHCSResponseCode
 */
tHCSResponseCode HCS_getDeviceFaultSatus(uint16_t *fltStatus);

#endif
