/*
 * Copyright (c) 2024, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

 /* Standard Includes */
#include <stdint.h>

/* TI includes */
#include "ti/driverlib/dl_spi.h"
#include "ti_msp_dl_config.h"

/* High-side Switch Includes */
#include <hcs_control_driver.h>
#include "current_sense.h"
#include "tps2hcs10_config.h"

/* Dummy function in case there is an error */
static void handleError(tHCSResponseCode resCode);

int main(void)
{
    uint16_t currentValue = 0;
    bool inLowCurrent = false;
    tHCSResponseCode resCode;

    /* Configuring SysConfig and DriverLib */
    SYSCFG_DL_init();
    NVIC_EnableIRQ(TIMER_0_INST_INT_IRQN);
    DL_SYSCTL_disableSleepOnExit();
    DL_TimerG_startCounter(TIMER_0_INST);

    /* Configuring the device initially */
    HCS_wakeupDevice();
    HCS_initializeDevice(&exportConfig);

    /* Enabling channel 1 initially */
    HCS_setSwitchState(0x01);

    while(1)
    {
        __WFI();

        /* Reading the load current value. We need to read the register twice
            as the */
        resCode = HCS_readRegister(TPS2HC10S_ADC_RESULT_CH1_I_REG,
                                    &currentValue);
        resCode.byte |=  HCS_readRegister(TPS2HC10S_ADC_RESULT_CH1_I_REG,
                                    &currentValue).byte;

        /* For each transaction, the high-side switch will return an error
            status code that will report common faults of the device. */
        if(resCode.byte != 0)
        {
            handleError(resCode);
        }

        /* Masking out the relevant bits */
        currentValue &= TPS2HC10S_ADC_RESULT_CH1_I_ADC_RESULT_CH1_I_MASK;

        /* Check to see if we are below the threshold where we want to turn
            on current sense scaling and change the KSNS ratio and enable
            scaling by 8x. This would allow for  */
        if((currentValue < LOW_CURRENT_SNS_THRESHOLD) &&
                (inLowCurrent == false))
        {
            exportConfig.diagConfigCh1.value.bits.OL_ON_EN_CH1 = 1;
            exportConfig.diagConfigCh1.value.bits.ISNS_SCALE_CH1 = 1;
            inLowCurrent = true;
            HCS_writeRegister(TPS2HC10S_DIAG_CONFIG_CH1_REG,
                                exportConfig.diagConfigCh1.value.word);

            /* Adding place to set a breakpoint for the sake of demonstration */
            asm ("nop");
        }
        else if((currentValue == LOW_CURRENT_SNS_SATURATION) &&
                    (inLowCurrent == true))
        {
            exportConfig.diagConfigCh1.value.bits.OL_ON_EN_CH1 = 0;
            exportConfig.diagConfigCh1.value.bits.ISNS_SCALE_CH1 = 0;
            HCS_writeRegister(TPS2HC10S_DIAG_CONFIG_CH1_REG,
                                exportConfig.diagConfigCh1.value.word);

            /* Adding place to set a breakpoint for the sake of demonstration */
            asm ("nop");
        }
    }

    return (0);
}

/* Dummy function that is called if the error return code is non-zero */
static void handleError(tHCSResponseCode resCode)
{
    /* Adding place to set a breakpoint for the sake of demonstration */
    asm ("nop");
}

/* Periodically wakes up the device every 1ms to read the current */
void TIMER_0_INST_IRQHandler(void)
{
    switch (DL_TimerG_getPendingInterrupt(TIMER_0_INST))
    {
        case DL_TIMER_IIDX_ZERO:
            break;
        default:
            break;
    }
}
