/*
 * Copyright (c) 2024, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

 /* Standard Includes */
#include <stdint.h>
#include <stdbool.h>

/* TI includes */
#include "ti/driverlib/dl_spi.h"
#include "ti_msp_dl_config.h"

/* High-side Switch Includes */
#include <hcs_control_driver.h>
#include "tps2hcs10_config.h"

/* Interrupt Signaling Functions */
volatile bool timerTriggered = false;

/* Dummy function in case there is an error */
static void handleError(tHCSResponseCode resCode);

int main(void)
{
    uint16_t currentValue = 0;
    bool inLowCurrent = false;
    tHCSResponseCode resCode;

    /* Configuring SysConfig and DriverLib */
    SYSCFG_DL_init();
    NVIC_EnableIRQ(TIMER_0_INST_INT_IRQN);
    NVIC_EnableIRQ(FLT_INT_GRP_INT_IRQN);
    DL_SYSCTL_disableSleepOnExit();

    /* Configuring the device initially */
    HCS_wakeupDevice();
    HCS_initializeDevice(&exportConfig);

    /* Enabling channel 1 initially */
    HCS_setSwitchState(0x01);

    while(1)
    {
        /* Wait for the fault line to trigger low on PB3 */
        __WFI();

        /* If we woke up from the interrupt, it means that the fault pin
            triggered. If it is an I2T interrupt, take the necessary steps to
            reset and re-enable */
        resCode = HCS_readRegister(TPS2HC10S_FLT_STAT_CH1_REG,
                                    &currentValue);
        resCode.byte |=  HCS_readRegister(TPS2HC10S_FLT_STAT_CH1_REG,
                                    &currentValue).byte;

        if(currentValue & TPS2HC10S_FLT_STAT_CH1_I2T_FLT_CH1_MASK)
        {
            /* Disabling the channel */
            HCS_setSwitchState(0);

            /* Setting the device to 2s retry state */
            exportConfig.i2tConfigCh1.value.bits.TCLDN_CH1 =
                (tcldn_ch1_en_3_0x2 >> TPS2HC10S_I2T_CONFIG_CH1_TCLDN_CH1_OFS);
            HCS_writeRegister(TPS2HC10S_I2T_CONFIG_CH1_REG,
                        exportConfig.i2tConfigCh1.value.word);

            /* Waiting for two seconds. At this point we would normally
                yield the tasks if we were in an RTOS, but just waiting for
                an interrupt here.  */

            DL_TimerA_startCounter(TIMER_0_INST);
            while(timerTriggered == false)
            {
                __WFI();
            }
            timerTriggered = false;

            /* Setting back to latch mode */
            exportConfig.i2tConfigCh1.value.bits.TCLDN_CH1 = 0;
            HCS_writeRegister(TPS2HC10S_I2T_CONFIG_CH1_REG,
                        exportConfig.i2tConfigCh1.value.word);

            /* Re-enabling the channel */
            HCS_setSwitchState(1);
        }

        /* For each transaction, the high-side switch will return an error
            status code that will report common faults of the device. */
        if(resCode.byte != 0)
        {
            handleError(resCode);
        }
    }

    return (0);
}

/* Dummy function that is called if the error return code is non-zero */
static void handleError(tHCSResponseCode resCode)
{
    /* Adding place to set a breakpoint for the sake of demonstration */
    asm ("nop");
}

/* Triggers when the fault pin triggers and wakes up from LPM */
 void GROUP1_IRQHandler(void)
{
    asm ("nop");
}

/* Used for the 2s delay timer for I2T trip cooldown */
void TIMER_0_INST_IRQHandler(void)
{
    timerTriggered = true;
}