/*
 * Copyright (c) 2024, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

 /* Standard Includes */
#include <stdint.h>
#include <stdbool.h>

/* TI includes */
#include "ti/driverlib/dl_spi.h"
#include "ti_msp_dl_config.h"

/* High-side Switch Includes */
#include <hcs_control_driver.h>
#include "tps2hcs10_config.h"

/* Interrupt Signaling Functions */
volatile tHCSResponseCode resCode;

/* Dummy function in case there is an error */
static void handleError(tHCSResponseCode resCode);

int main(void)
{
    uint16_t currentValue = 0;

    /* Configuring SysConfig and DriverLib */
    SYSCFG_DL_init();
    DL_SYSCTL_disableSleepOnExit();

    /* Configuring the device initially */
    HCS_wakeupDevice();
    HCS_initializeDevice(&exportConfig);

    /* Enabling the interrupt */
    NVIC_EnableIRQ(FLT_INT_GRP_INT_IRQN);

    /* Enabling channel 1 initially */
    HCS_setSwitchState(0x01);

    while(1)
    {
        /* Putting the device into LPM. 800mA exit current on CH1 */
        HCS_gotoLPM(lpm_exit_curr_ch1_en_2_0x1, lpm_exit_curr_ch2_en_1_0x0);

        /* Wait for the fault line to trigger low on PB3 */
        __WFI();

        /* If we woke up from the interrupt, check to make sure it was a signal
            for an LPM wakeup. The idea here is that the user increases the
            load current somehow to "force the device" from LPM. */
        resCode = HCS_readRegister(TPS2HC10S_FLT_STAT_CH1_REG,
                                    &currentValue);
        resCode.byte |=  HCS_readRegister(TPS2HC10S_FLT_STAT_CH1_REG,
                                    &currentValue).byte;

        if(currentValue & TPS2HC10S_FLT_STAT_CH1_LPM_WAKE_CH1_MASK)
        {
            /* Set a breakpoint here for demonstration */
            asm ("nop");
        }
    }

    return (0);
}

/* Dummy function that is called if the error return code is non-zero */
static void handleError(tHCSResponseCode resCode)
{
    /* Adding place to set a breakpoint for the sake of demonstration */
    asm ("nop");
}

/* Triggers when the fault pin triggers and wakes up from LPM */
 void GROUP1_IRQHandler(void)
{
    asm ("nop");
}
